import click
import paramiko
from paramiko.client import SSHClient
from paramiko.rsakey import RSAKey

from aws.cli.utils import exec_command, run_evolution_command, key_file_path, username
from src.utils.logger import logger

log = logger()


@click.command()
@click.option("--instance-ip", required=True, help="IP address of EC2 instance")
@click.option("--username", default=username, help="Username of a user to log in with")
@click.option("--key-file-path", default=key_file_path, help="Path to SSH key file")
@click.option(
    "--command", default=run_evolution_command, help="Command to start the evolution"
)
def run_evolution(instance_ip, username, key_file_path, command):
    log.info(f"Connecting to: {instance_ip}")

    ssh = SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    key = RSAKey.from_private_key_file(key_file_path)
    ssh.connect(hostname=instance_ip, username=username, pkey=key)
    log.info(f"Connected!")

    log.info(f"Run evolution: {command}")
    exec_command(ssh, log, command)

    log.info("All done!")


if __name__ == "__main__":
    run_evolution()
