import pandas as pd
from experiments.shared.stats import complete_anova_run


if __name__ == "__main__":
    # NOTE: Select results csv
    csv_path = "transformer_results/20220526/transformer_25-05-2022_20:46:02.csv"
    df = pd.read_csv(csv_path)

    # One way ANOVA, only test dependency on attention model
    # The p value obtained from ANOVA analysis is significant (p < 0.05)
    # Note: p-value 0.001 from tukey_hsd output should be interpreted as <=0.001
    # Example formula: anova_model = "value~C(Genotype)+C(years)+C(Genotype):C(years)"
    formula_mae = "mae ~ C(transformer_type)"
    formula_rmse = "rmse ~ C(transformer_type)"

    df_mae = df[["transformer_type", "dataset_horizon", "mae"]]
    df_rmse = df[["transformer_type", "dataset_horizon", "rmse"]]

    (anova_df_mae, tukey_df_mae, normality_mae, homogeneity_mae,) = complete_anova_run(
        df_mae,
        formula_mae,
        "mae",
        "transformer_type",
    )

    (
        anova_df_rmse,
        tukey_df_rmse,
        normality_rmse,
        homogeneity_rmse,
    ) = complete_anova_run(
        df_rmse,
        formula_rmse,
        "rmse",
        "transformer_type",
    )

    print(">>> Transformer types")
    print(anova_df_mae)
    print(anova_df_rmse)
