import copy
import random
import unittest

from experiments.e1.src.genotype import Genotype, randomize_genotype


def random_parameter_mutation(genotype: Genotype):
    genotype.mark_dirty()  # Important to reset fitness

    random_index = random.choice(range(genotype.chromosome_length()))
    parameter = genotype.chromosome(random_index)

    allowed_values = copy.deepcopy(parameter.get_allowed_values())
    allowed_values.remove(parameter.get_value())  # Remove the value currently used

    value = random.choice(allowed_values)
    parameter.set_value(value)


def one_point_crossover(genotype1: Genotype, genotype2: Genotype):
    possible_points = list(range(1, genotype1.chromosome_length()))
    point_index = random.choice(possible_points)

    chromosome1 = copy.deepcopy(genotype1.chromosome())
    chromosome2 = copy.deepcopy(genotype2.chromosome())
    left1, right1 = chromosome1[:point_index], chromosome1[point_index:]
    left2, right2 = chromosome2[:point_index], chromosome2[point_index:]

    # Fitness is reset when creating a new genotype
    return Genotype(left1 + right2), Genotype(left2 + right1)


def uniform_crossover(genotype1: Genotype, genotype2: Genotype):
    chromosome_length = genotype1.chromosome_length()
    chromosome1 = copy.deepcopy(genotype1.chromosome())
    chromosome2 = copy.deepcopy(genotype2.chromosome())

    new_chromosome1 = []
    new_chromosome2 = []
    for i in range(chromosome_length):
        if random.random() < 0.5:
            new_chromosome1.append(chromosome1[i])
            new_chromosome2.append(chromosome2[i])
        else:
            new_chromosome1.append(chromosome2[i])
            new_chromosome2.append(chromosome1[i])

    # Fitness is reset when creating a new genotype
    return Genotype(new_chromosome1), Genotype(new_chromosome2)


class TestOperators(unittest.TestCase):
    def test_one_point_crossover(self):
        g1 = Genotype.create_default()
        randomize_genotype(g1)

        g2 = Genotype.create_default()
        randomize_genotype(g2)

        g11, g22 = one_point_crossover(g1, g2)
        self.assertEqual(g1.chromosome_length(), g11.chromosome_length())
        self.assertEqual(g2.chromosome_length(), g22.chromosome_length())

    def test_uniform_crossover(self):
        g1 = Genotype.create_default()
        randomize_genotype(g1)

        g2 = Genotype.create_default()
        randomize_genotype(g2)

        g11, g22 = uniform_crossover(g1, g2)
        self.assertEqual(g1.chromosome_length(), g11.chromosome_length())
        self.assertEqual(g2.chromosome_length(), g22.chromosome_length())
