import random
import torch
from torch import optim, nn
from torch.utils.data import DataLoader

from experiments.e4.nbeats.nbeats import NeuralBeats
from experiments.e4.utils.visualisation import plot_prediction
from src.dataset.SyntheticDataset import SyntheticDataset, SeriesType
from src.utils.seed import temp_seed

# https://towardsdatascience.com/building-rnn-lstm-and-gru-for-time-series-using-pytorch-a46e5b094e7b


def train(batch_size, input_dim, output_dim, train_set, test_set, epochs, device):
    model = NeuralBeats(
        forecast_length=output_dim,
        backcast_length=input_dim,
        batch_size=batch_size,
        device=device,
    )
    loss_fn = nn.SmoothL1Loss(reduction="sum")
    optimizer = optim.Adam(model.net_parameters(), lr=1e-3, weight_decay=1e-6)

    model.fit(
        train_set,
        test_set,
        batch_size,
        epochs=epochs,
        loss_fn=loss_fn,
        optimizer=optimizer,
        plot=False,
    )
    return model


def make_prediction(model, source):
    predictions = model.predict(source)
    return predictions


def main():
    input_dim = 200
    output_dim = 50
    batch_size = 16
    epochs = 20

    device = "cuda" if torch.cuda.is_available() else "cpu"

    dataset = SyntheticDataset(
        SeriesType.COMPOSED_WITH_NOISE, x_length=input_dim, y_length=output_dim
    )
    with temp_seed(1):
        train_set, test_set = torch.utils.data.random_split(dataset, [4000, 750])

    # Shuffle datasets!
    with temp_seed(1):
        train_set = [train_set[i] for i in range(len(train_set))]
        random.shuffle(train_set)
        test_set = [test_set[i] for i in range(len(test_set))]
        random.shuffle(test_set)

    model = train(
        batch_size, input_dim, output_dim, train_set, test_set, epochs, device
    )

    example = dataset[0]
    source = example["x"]
    target = example["y"]
    predictions = make_prediction(model, source)
    plot_prediction(source, target, predictions)


if __name__ == "__main__":
    main()
