import torch
from torch import nn

from src.training.NoamOptimizer import NoamOptimizer
from src.training.TransformerLossCompute import TransformerLossCompute
from src.transformers.encoder_only.TimeSeriesEncoderTransformer import (
    TimeSeriesEncoderTransformer,
)
from src.transformers.encoder_only.train_encoder_only import run_epoch, batch_iterator


def train(
    parameters, batch_size, input_dim, output_dim, train_set, test_set, epochs, device
):
    embedding_dim = parameters[0]
    patch_size = parameters[1]
    head_count = parameters[2]
    layer_count = parameters[3]
    forward_expansion = parameters[4]
    dropout = parameters[5]

    model = TimeSeriesEncoderTransformer.create(
        layers=layer_count,
        embedding_dim=embedding_dim,
        input_dim=input_dim,
        output_dim=output_dim,
        head_count=head_count,
        forward_expansion=forward_expansion,
        dropout=dropout,
        patch_size=patch_size,
    ).to(device)

    criterion = nn.SmoothL1Loss(reduction="sum")
    optimizer = torch.optim.AdamW(model.parameters(), lr=0, betas=(0.9, 0.98), eps=1e-9)
    scheduled_optimizer = NoamOptimizer(
        optimizer,
        embedding_dim,
        1000,
    )

    # Train
    for epoch in range(epochs):
        print(f"### Epoch: {epoch + 1} ###")

        model.train()
        train_loss = run_epoch(
            batch_iterator(train_set, batch_size, input_dim, device),
            model,
            TransformerLossCompute(model.regression, criterion, scheduled_optimizer),
            patch_size,
        )
        print(f"## Train acc loss: {train_loss} ##")

        model.eval()
        eval_loss = run_epoch(
            batch_iterator(test_set, batch_size, input_dim, device),
            model,
            TransformerLossCompute(model.regression, criterion, None),
            patch_size,
        )
        print(f"## Eval acc loss: {eval_loss} ##")

    return model
