import copy
from enum import Enum
from typing import List

from src.components.attention.AdaptiveSpanAttention import (
    AdaptiveSpanAttention,
    AdaptiveSpanAttention1,
)
from src.components.attention.AttentionFreeFullAttention import (
    AttentionFreeFullAttention,
    AttentionFreeFullAttention1,
)
from src.components.attention.AttentionFreeLocalAttention import (
    AttentionFreeLocalAttention,
    AttentionFreeLocalAttention1,
)
from src.components.attention.AttentionFreeSimpleAttention import (
    AttentionFreeSimpleAttention,
    AttentionFreeSimpleAttention1,
)
from src.components.attention.AttentionPassthrough import (
    AttentionPassthrough,
    AttentionPassthrough1,
)
from src.components.attention.BigBirdBlockSparseAttention import (
    BigBirdBlockSparseAttention,
    BigBirdBlockSparseAttention1,
)
from src.components.attention.ConformerConvolutionAttention import (
    ConformerConvolutionAttention,
    ConformerConvolutionAttention1,
)
from src.components.attention.FNetFourierTransformAttention import (
    FNetFourierTransformAttention,
    FNetFourierTransformAttention1,
)
from src.components.attention.FactorizedDenseAttentionSynthesizer import (
    FactorizedDenseAttentionSynthesizer,
    FactorizedDenseAttentionSynthesizer1,
)
from src.components.attention.FactorizedRandomAttentionSynthesizer import (
    FactorizedRandomAttentionSynthesizer,
    FactorizedRandomAttentionSynthesizer1,
)
from src.components.attention.FastWeightsAttention import (
    FastWeightsAttention,
    FastWeightsAttention1,
)
from src.components.attention.FavorAttentionPerformer import (
    FavorAttentionPerformer,
    FavorAttentionPerformer1,
)
from src.components.attention.KmeansRoutingAttention import (
    KmeansRoutingAttention,
    KmeansRoutingAttention1,
)
from src.components.attention.LambdaLayerAttention import (
    LambdaLayerAttention,
    LambdaLayerAttention1,
)
from src.components.attention.LinearAttentionTransformersRNNs import (
    LinearAttentionTransformersRNNs,
    LinearAttentionTransformersRNNs1,
)
from src.components.attention.LinformerAttention import (
    LinformerAttention,
    LinformerAttention1,
)
from src.components.attention.LocalAttention import LocalAttention, LocalAttention1
from src.components.attention.LocalitySensitiveHashingAttentionReformer import (
    LocalitySensitiveHashingAttentionReformer,
    LocalitySensitiveHashingAttentionReformer1,
)
from src.components.attention.LongShortAttention import (
    LongShortAttention,
    LongShortAttention1,
)
from src.components.attention.LongformerAttention import (
    LongformerAttention,
    LongformerAttention1,
)
from src.components.attention.RectifiedLinearAttention import (
    RectifiedLinearAttention,
    RectifiedLinearAttention1,
)
from src.components.attention.RelativeMultiHeadAttentionXL import (
    RelativeMultiHeadAttentionXL,
    RelativeMultiHeadAttentionXL1,
)
from src.components.attention.ScaledDotProductMultiHeadAttention import (
    ScaledDotProductMultiHeadAttention,
    ScaledDotProductMultiHeadAttention1,
)
from src.components.attention.TupeMultiHeadAttention import (
    TupeMultiHeadAttention,
    TupeMultiHeadAttention1,
)


class Attention(str, Enum):
    AdaptiveSpanAttention = ("adaptive_span_attention",)
    AttentionFreeFullAttention = ("attention_free_full_attention",)
    AttentionFreeLocalAttention = ("attention_free_local_attention",)
    AttentionFreeSimpleAttention = ("attention_free_simple_attention",)
    BigBirdBlockSparseAttention = ("big_bird_block_sparse_attention",)
    ConformerConvolutionAttention = ("conformer_convolution_attention",)
    FactorizedDenseAttention = ("factorized_dense_attention",)
    FactorizedRandomAttention = ("factorized_random_attention",)
    FastWeightsAttention = ("fast_weights_attention",)
    FavorAttentionPerformer = ("favor_attention_performer",)
    FNetFourierTransformAttention = ("fnet_fourier_transform_attention",)
    KmeansRoutingAttention = ("kmeans_routing_attention",)
    LambdaLayerAttention = ("lambda_layer_attention",)
    LinearAttentionTransformersRNNs = ("linear_attention_transformer_rnns",)
    LinformerAttention = ("linformer_attention",)
    LocalAttention = ("local_attention",)
    LocalitySensitiveHashingAttentionReformer = (
        "locality_sensitive_hashing_attention_reformer",
    )
    LongformerAttention = ("longformer_attention",)
    LongShortAttention = ("long_short_attention",)
    RectifiedLinearAttention = ("rectified_linear_attention",)
    RelativeMultiHeadAttentionXL = ("relative_multihead_attention",)
    VanillaAttention = ("vanilla_attention",)
    TupeMultiHeadAttention = ("tupe_multihead_attention",)
    Passthrough = ("passthrough_attention",)

    @staticmethod
    def module_list() -> List[Enum]:
        return [attention for attention in Attention]

    @staticmethod
    def module_string_list() -> List[str]:
        return [attention.value for attention in Attention]

    def module(self, embedding_dim, head_count, dropout, input_dim, batch_size):
        registry = {
            Attention.AdaptiveSpanAttention: AdaptiveSpanAttention(
                embedding_dim, head_count, input_dim
            ),
            Attention.AttentionFreeFullAttention: AttentionFreeFullAttention(
                embedding_dim, head_count
            ),
            Attention.AttentionFreeLocalAttention: AttentionFreeLocalAttention(
                embedding_dim, input_dim
            ),
            Attention.AttentionFreeSimpleAttention: AttentionFreeSimpleAttention(
                embedding_dim, head_count
            ),
            Attention.BigBirdBlockSparseAttention: BigBirdBlockSparseAttention(
                embedding_dim, input_dim, head_count
            ),
            Attention.ConformerConvolutionAttention: ConformerConvolutionAttention(
                embedding_dim, dropout=dropout
            ),
            Attention.FactorizedDenseAttention: FactorizedDenseAttentionSynthesizer(
                embedding_dim, head_count, dropout
            ),
            Attention.FactorizedRandomAttention: FactorizedRandomAttentionSynthesizer(
                embedding_dim, head_count, dropout, batch_size
            ),
            Attention.FastWeightsAttention: FastWeightsAttention(
                embedding_dim, head_count
            ),
            Attention.FavorAttentionPerformer: FavorAttentionPerformer(
                embedding_dim, head_count, dropout
            ),
            Attention.FNetFourierTransformAttention: FNetFourierTransformAttention(
                embedding_dim, head_count
            ),
            Attention.KmeansRoutingAttention: KmeansRoutingAttention(
                embedding_dim, input_dim, head_count
            ),
            Attention.LambdaLayerAttention: LambdaLayerAttention(
                embedding_dim, head_count, input_dim, dropout
            ),
            Attention.LinearAttentionTransformersRNNs: LinearAttentionTransformersRNNs(
                embedding_dim, head_count
            ),
            Attention.LinformerAttention: LinformerAttention(
                embedding_dim, head_count, input_dim
            ),
            Attention.LocalAttention: LocalAttention(
                embedding_dim, head_count, dropout
            ),
            Attention.LocalitySensitiveHashingAttentionReformer: LocalitySensitiveHashingAttentionReformer(
                embedding_dim, input_dim, head_count, dropout
            ),
            Attention.LongformerAttention: LongformerAttention(
                embedding_dim, input_dim, head_count
            ),
            Attention.LongShortAttention: LongShortAttention(
                embedding_dim, input_dim, head_count, dropout
            ),
            Attention.RectifiedLinearAttention: RectifiedLinearAttention(
                embedding_dim, head_count
            ),
            Attention.RelativeMultiHeadAttentionXL: RelativeMultiHeadAttentionXL(
                embedding_dim, head_count
            ),
            Attention.VanillaAttention: ScaledDotProductMultiHeadAttention(
                embedding_dim, head_count
            ),
            Attention.TupeMultiHeadAttention: TupeMultiHeadAttention(
                embedding_dim, head_count, dropout
            ),
            Attention.Passthrough: AttentionPassthrough(),
        }
        return copy.deepcopy(registry[self])

    def module1(self, embedding_dim, head_count, dropout, input_dim, batch_size):
        registry = {
            Attention.AdaptiveSpanAttention: AdaptiveSpanAttention1(
                embedding_dim, head_count, input_dim
            ),
            Attention.AttentionFreeFullAttention: AttentionFreeFullAttention1(
                embedding_dim, head_count
            ),
            Attention.AttentionFreeLocalAttention: AttentionFreeLocalAttention1(
                embedding_dim, input_dim
            ),
            Attention.AttentionFreeSimpleAttention: AttentionFreeSimpleAttention1(
                embedding_dim, head_count
            ),
            Attention.BigBirdBlockSparseAttention: BigBirdBlockSparseAttention1(
                embedding_dim, input_dim, head_count
            ),
            Attention.ConformerConvolutionAttention: ConformerConvolutionAttention1(
                embedding_dim, dropout=dropout
            ),
            Attention.FactorizedDenseAttention: FactorizedDenseAttentionSynthesizer1(
                embedding_dim, head_count, dropout
            ),
            Attention.FactorizedRandomAttention: FactorizedRandomAttentionSynthesizer1(
                embedding_dim, head_count, dropout, batch_size
            ),
            Attention.FastWeightsAttention: FastWeightsAttention1(
                embedding_dim, head_count
            ),
            Attention.FavorAttentionPerformer: FavorAttentionPerformer1(
                embedding_dim, head_count, dropout
            ),
            Attention.FNetFourierTransformAttention: FNetFourierTransformAttention1(
                embedding_dim, head_count
            ),
            Attention.KmeansRoutingAttention: KmeansRoutingAttention1(
                embedding_dim, input_dim, head_count
            ),
            Attention.LambdaLayerAttention: LambdaLayerAttention1(
                embedding_dim, head_count, input_dim, dropout
            ),
            Attention.LinearAttentionTransformersRNNs: LinearAttentionTransformersRNNs1(
                embedding_dim, head_count
            ),
            Attention.LinformerAttention: LinformerAttention1(
                embedding_dim, head_count, input_dim
            ),
            Attention.LocalAttention: LocalAttention1(
                embedding_dim, head_count, dropout
            ),
            Attention.LocalitySensitiveHashingAttentionReformer: LocalitySensitiveHashingAttentionReformer1(
                embedding_dim, input_dim, head_count, dropout
            ),
            Attention.LongformerAttention: LongformerAttention1(
                embedding_dim, input_dim, head_count
            ),
            Attention.LongShortAttention: LongShortAttention1(
                embedding_dim, input_dim, head_count, dropout
            ),
            Attention.RectifiedLinearAttention: RectifiedLinearAttention1(
                embedding_dim, head_count
            ),
            Attention.RelativeMultiHeadAttentionXL: RelativeMultiHeadAttentionXL1(
                embedding_dim, head_count
            ),
            Attention.VanillaAttention: ScaledDotProductMultiHeadAttention1(
                embedding_dim, head_count
            ),
            Attention.TupeMultiHeadAttention: TupeMultiHeadAttention1(
                embedding_dim, head_count, dropout
            ),
            Attention.Passthrough: AttentionPassthrough1(),
        }
        return copy.deepcopy(registry[self])
