from typing import Optional

import torch
from torch import nn

# Fourier transform attention
# Implementation heavily inspired by: https://nn.labml.ai/transformers/fnet/index.html
# Paper: FNet: Mixing Tokens with Fourier Transforms


class FNetFourierTransformAttention(nn.Module):
    def __init__(self, embedding_dim: int, head_count: int):
        super(FNetFourierTransformAttention, self).__init__()
        self.attention = FNetFourierTransformAttention1(embedding_dim, head_count)

    def forward(self, x, mask=None):
        return self.attention(x, x, x, mask)


class FNetFourierTransformAttention1(nn.Module):
    """Fourier transform attention"""

    def __init__(self, embedding_dim: int, head_count: int):
        super(FNetFourierTransformAttention1, self).__init__()

    # NOTE: Masking is not supported!
    def forward(
        self,
        query: torch.Tensor,
        key: torch.Tensor,
        value: torch.Tensor,
        mask: Optional[torch.Tensor] = None,
    ):
        x = query

        # Apply the Fourier transform along the embedding dimension
        fft_hidden = torch.fft.fft(x, dim=2)
        # Apply the Fourier transform along the sequence dimension
        fft_seq = torch.fft.fft(fft_hidden, dim=1)

        # The output of the Fourier transform is a tensor of complex numbers
        # Get the real component
        reals = torch.real(fft_seq)
        return reals
