import unittest

import torch
from torch import nn


# NOTE: Dilations >1 turn the module into dilated convolution
class Conv2d(nn.Module):
    def __init__(
        self,
        kernel_width: int = 3,
        kernel_height: int = 3,
        dilation_width: int = 1,
        dilation_height: int = 1,
    ):
        super(Conv2d, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_width % 2) == 0:
            kernel_width = kernel_width + 1
        if (kernel_height % 2) == 0:
            kernel_height = kernel_height + 1

        self.conv2d = nn.Conv2d(
            1,
            1,
            (kernel_height, kernel_width),
            stride=1,
            padding=(
                (kernel_height // 2) * dilation_height,
                (kernel_width // 2) * dilation_width,
            ),
            dilation=(dilation_height, dilation_width),
        )

    # Run convolution along sequence_length and embedding_dim
    def forward(self, x):
        x = x.unsqueeze(1)
        out = self.conv2d(x)
        out = out.squeeze(1)
        return out


class TestConv2d(unittest.TestCase):
    def test(self):
        conv1d = Conv2d(16, 5, 7)
        input = torch.randn(20, 100, 16)
        output = conv1d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
