import unittest

import torch
from torch import nn


class MaxPool1d(nn.Module):
    def __init__(self, kernel_size: int = 3):
        super(MaxPool1d, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_size % 2) == 0:
            kernel_size = kernel_size + 1

        self.pool = nn.MaxPool1d(kernel_size, stride=1, padding=kernel_size // 2)

    # Run pooling along embedding_dim
    def forward(self, x):
        sequence_length = x.size(1)
        out = torch.zeros(x.shape, device=x.device)
        for i in range(sequence_length):
            pool_in = x[:, i, :].unsqueeze(1)
            pool_out = self.pool(pool_in)
            out[:, i, :] = pool_out.squeeze(1)
        return out


class TestMaxPool1d(unittest.TestCase):
    def test(self):
        pool1d = MaxPool1d(16, 5)
        input = torch.randn(20, 100, 16)
        output = pool1d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
