import unittest

import torch
from torch import nn


class MaxPool2d(nn.Module):
    def __init__(self, kernel_width: int = 3, kernel_height: int = 3):
        super(MaxPool2d, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_width % 2) == 0:
            kernel_width = kernel_width + 1
        if (kernel_height % 2) == 0:
            kernel_height = kernel_height + 1

        self.pool = nn.MaxPool2d(
            (kernel_height, kernel_width),
            stride=1,
            padding=(kernel_height // 2, kernel_width // 2),
        )

    # Run pooling along sequence_length and embedding_dim
    def forward(self, x):
        x = x.unsqueeze(1)
        out = self.pool(x)
        out = out.squeeze(1)
        return out


class TestConv2d(unittest.TestCase):
    def test(self):
        pool2d = MaxPool2d(16, 5, 7)
        input = torch.randn(20, 100, 16)
        output = pool2d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
