import unittest

import torch
import torch.nn as nn


class Dropout2d(nn.Module):
    def __init__(self, dropout):
        super(Dropout2d, self).__init__()
        self.probability = dropout
        self.dropout = nn.Dropout2d(dropout)

    def forward(self, x):
        # x: (N, C, H, W)
        # Randomly zero out entire channels -> consider input samples (i.e. input size) channels
        x = x.unsqueeze(2)
        out = self.dropout(x)
        out = out.squeeze(2)
        return out


class TestDropout2d(unittest.TestCase):
    def test(self):
        dropout = Dropout2d(0.5)
        input = torch.randn(20, 100, 16)
        output = dropout(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
