import torch.nn as nn


# For patch_size=1 this one is equal to PointEmbedding
class TimeSeriesPatchEmbedding(nn.Module):
    def __init__(self, embedding_dim, patch_size):
        super(TimeSeriesPatchEmbedding, self).__init__()
        self.embedding_dim = embedding_dim
        self.patch_size = patch_size
        self.linear = nn.Linear(patch_size, embedding_dim)

    def fix_weights(self, value=0.1):
        nn.init.constant_(self.linear.weight, value)

    def forward(self, x):
        # x: (batch_size, sequence_length)
        # output: (batch_size, sequence_length, embedding_size)
        x = x.view(x.size(0), -1, self.patch_size)
        output = self.linear(x)
        return output
