import random
import unittest

import numpy as np
import torch
import torch.nn as nn


class TimeSeriesPointEmbedding(nn.Module):
    def __init__(self, embedding_dim):
        super(TimeSeriesPointEmbedding, self).__init__()
        self.embedding_dim = embedding_dim
        self.linear = nn.Linear(1, embedding_dim)

    def fix_weights(self, value=0.1):
        nn.init.constant_(self.linear.weight, value)

    def forward(self, x):
        # x: (batch_size, sequence_length)
        # output: (batch_size, sequence_length, embedding_size)
        x = x.view(x.size(0), x.size(1), 1)
        output = self.linear(x)
        return output


class TestTimeSeriesPointEmbedding(unittest.TestCase):
    def test(self):
        random.seed(0)
        torch.manual_seed(0)

        data = np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], dtype=np.dtype("f4"))
        x = torch.tensor(data)
        print(x.size())
        print(x)

        embedding = TimeSeriesPointEmbedding(2)
        embedding.fix_weights()
        y = embedding.forward(x)
        print(f"\n{y.size()}")
        print(y)

        output = y.detach().numpy()
        expected = np.array(
            [
                [[-0.7230, -0.6359], [-0.6230, -0.5359], [-0.5230, -0.4359]],
                [[-0.4230, -0.3359], [-0.3230, -0.2359], [-0.2230, -0.1359]],
            ]
        )
        np.testing.assert_array_almost_equal(output, expected, decimal=3)
