import copy
import torch
from torch import nn

from src.components.transformer.ResidualConnection import ResidualConnection
from src.components.memory.ResidualConnectionWithMemory import (
    ResidualConnectionWithMemory,
)

# Transformer XL layer with memory
# Heavily inspired by: https://nn.labml.ai/transformers/xl/index.html


class TransformerWithMemoryXL(nn.Module):
    def __init__(self, layer, layer_count):
        super(TransformerWithMemoryXL, self).__init__()
        self.layers = nn.ModuleList([copy.deepcopy(layer) for _ in range(layer_count)])
        self.norm = nn.LayerNorm(layer.embedding_dim)
        self.memory = []

    def forward(self, x, mask):
        self.memory = []

        for i, layer in enumerate(self.layers):
            self.memory.append(x.detach())
            if i - 1 >= 0:
                mem = self.memory[i - 1]
            else:
                mem = None
            x = layer(x, mask, mem)

        return self.norm(x)


class TransformerLayerWithMemoryXL(nn.Module):
    def __init__(self, embedding_dim, self_attention, feed_forward, dropout):
        super(TransformerLayerWithMemoryXL, self).__init__()
        self.embedding_dim = embedding_dim
        self.self_attention = self_attention
        self.feed_forward = feed_forward
        self.residual1 = ResidualConnectionWithMemory(embedding_dim, dropout)
        self.residual2 = ResidualConnection(embedding_dim, dropout)

    # TODO: It should be possible to have access to more than 1 memory step (the last one currently)
    def forward(self, x, mask, memory):
        if memory is not None:
            m_z = torch.cat((memory, x), dim=1)
        else:
            m_z = x

        attention = self.residual1(
            x, m_z, lambda x, m_z: self.self_attention(x, m_z, m_z, mask)
        )
        return self.residual2(attention, self.feed_forward)
