from torch import nn

# AdaNorm, a normalization technique without learnable parameters
# Adapted from: https://github.com/lancopku/AdaNorm/blob/master/machine%20translation/fairseq/modules/layer_norm.py
# Paper: Understanding and Improving Layer Normalization


class AdaNorm(nn.Module):
    def __init__(self, adanorm_scale=1.0, eps=1e-6):
        super(AdaNorm, self).__init__()
        self.adanorm_scale = adanorm_scale
        self.eps = eps

    def forward(self, x):
        mean = x.mean(-1, keepdim=True)
        std = x.std(-1, keepdim=True)

        input = x - mean
        mean = input.mean(-1, keepdim=True)
        gra_norm = (1 / 10 * (input - mean) / (std + self.eps)).detach()
        input_norm = (input - input * gra_norm) / (std + self.eps)
        return input_norm * self.adanorm_scale
