import torch
from torch import nn

# Replacement of LN module i.e. scaled l2 normalization
# Adapted from: https://github.com/tnq177/transformers_without_tears/blob/master/layers.py
# Paper: Transformers without Tears: Improving the Normalization of Self-Attention


class ScaledL2Normalization(nn.Module):
    def __init__(self, scale=1.0, eps=1e-6):
        super(ScaledL2Normalization, self).__init__()
        self.scale = scale
        self.eps = eps

    def forward(self, x):
        norm = self.scale / torch.norm(x, dim=-1, keepdim=True).clamp(min=self.eps)
        return x * norm
