from torch import nn

# GradNorm, a normalization technique without learnable parameters
# Adapted from: https://github.com/lancopku/AdaNorm/blob/master/machine%20translation/fairseq/modules/layer_norm.py
# Paper: GradNorm: Gradient Normalization for Adaptive Loss Balancing in Deep Multitask Networks


class GradNorm(nn.Module):
    def __init__(self, eps=1e-6):
        super(GradNorm, self).__init__()
        self.eps = eps

    def forward(self, x):
        mean = x.mean(dim=-1, keepdim=True)
        std = x.std(dim=-1, keepdim=True)
        input_norm = (x - mean) / (std + self.eps)
        output = x.detach() + input_norm
        return output
