import torch
from torch import nn


class SimpleAbsolutePositionalEncoding(nn.Module):
    def __init__(self, embedding_dim: int, dropout: float):
        super(SimpleAbsolutePositionalEncoding, self).__init__()
        self.embedding_dim = embedding_dim
        self.dropout = nn.Dropout(dropout)

    def forward(self, x):
        batch_size = x.size(0)
        sequence_length = x.size(1)
        encoding = (
            torch.linspace(0, 1, sequence_length)
            .unsqueeze(0)
            .unsqueeze(-1)
            .expand(batch_size, sequence_length, self.embedding_dim)
            .to(device=x.device)
        )
        result = self.dropout(encoding) + x
        return result
