import torch
from torch import nn


# Should be injected into attention matrix, not concatenated with embedding!
# This had great results in combination with absolute positional encoding!
# Usually used for keys and queries
class RotaryPositionalEncoding(nn.Module):
    def __init__(self, d_k: int, base: int = 10000):
        super().__init__()
        self.theta = nn.Parameter(
            1.0 / (base ** (torch.arange(0, d_k, 2).float() / d_k)), requires_grad=False
        )

    def forward(self, x):
        # HUGE NOTE: d has to be divisible by 2! If its 1 you are doomed!
        _, _, sequence_length, d = x.shape
        d_half = d // 2

        seq_idx = torch.arange(sequence_length, device=x.device).type_as(self.theta)
        idx_theta = torch.einsum("n,d->nd", seq_idx, self.theta)
        idx_theta2 = torch.cat([idx_theta, idx_theta], dim=1)
        negative_half_x = torch.cat([-x[:, :, :, d_half:], x[:, :, :, :d_half]], dim=-1)

        rx = (x * idx_theta2.cos()[None, None, :, :]) + (
            negative_half_x * idx_theta2.sin()[None, None, :, :]
        )
        return rx
