import copy
import torch.nn as nn

from src.components.transformer.ResidualConnection import ResidualConnection


class Decoder(nn.Module):
    def __init__(self, decoder_layer, layer_count):
        super(Decoder, self).__init__()
        self.layers = nn.ModuleList(
            [copy.deepcopy(decoder_layer) for _ in range(layer_count)]
        )
        self.norm = nn.LayerNorm(decoder_layer.embedding_dim)

    def forward(self, x, memory, source_mask, target_mask):
        for layer in self.layers:
            x = layer(x, memory, source_mask, target_mask)
        return self.norm(x)


class DecoderLayer(nn.Module):
    def __init__(
        self, embedding_dim, self_attention, encoder_attention, feed_forward, dropout
    ):
        super(DecoderLayer, self).__init__()
        self.embedding_dim = embedding_dim
        self.self_attention = self_attention
        self.encoder_attention = encoder_attention
        self.feed_forward = feed_forward
        self.residual1 = ResidualConnection(embedding_dim, dropout)
        self.residual2 = ResidualConnection(embedding_dim, dropout)
        self.residual3 = ResidualConnection(embedding_dim, dropout)

    def forward(self, x, memory, source_mask, target_mask):
        x = self.residual1(x, lambda x: self.self_attention(x, x, x, target_mask))
        x = self.residual2(
            x, lambda x: self.encoder_attention(x, memory, memory, source_mask)
        )
        return self.residual3(x, self.feed_forward)
