import unittest
from typing import TypeVar, List

from src.evolutionary.genotype.parameters.IParameter import IParameter

T = TypeVar("T", int, str, float)


class ConstantParameter(IParameter):
    def __init__(self, value: T):
        self.value: T = value
        self.allowed_values: List[T] = [value]
        self.type = "constant"

    def get_value(self):
        return self.value

    def set_value(self, value: T):
        if value not in self.allowed_values:
            raise ValueError("Value not allowed")
        self.value = value

    def get_allowed_values(self) -> List[T]:
        return self.allowed_values

    # Serialize/deserialize
    def to_dict(self) -> dict:
        data = {
            "value": self.value,
            "allowed_values": self.allowed_values,
            "type": self.type,
        }
        return data

    @staticmethod
    def from_dict(data: dict):
        value = data["value"]
        return ConstantParameter(value)


class TestConstantParameter(unittest.TestCase):
    def test(self):
        parameter = ConstantParameter(0)
        self.assertEqual(parameter.get_allowed_values(), [0])
        self.assertEqual(parameter.get_value(), 0)
