from enum import Enum
from typing import List

from src.evolutionary.genotype.parameters.ConstantParameter import ConstantParameter
from src.evolutionary.genotype.parameters.NeighbourhoodParameter import (
    NeighbourhoodParameter,
)
from src.evolutionary.genotype.parameters.RangeParameter import RangeParameter
from src.evolutionary.genotype.parameters.SetParameter import SetParameter


class ParameterType(str, Enum):
    Constant = "constant"
    Neighbourhood = "neighbourhood"
    Range = "range"
    Set = "set"

    @staticmethod
    def from_str(string: str):
        return ParameterType(string)

    @staticmethod
    def type_list() -> List[Enum]:
        return [ty for ty in ParameterType]

    def instance(
        self,
        value,
        allowed_values=None,
        allowed_range=None,
        value_count=None,
        step=None,
        limits=None,
    ):
        if self == ParameterType.Constant:
            return ConstantParameter(value)
        elif self == ParameterType.Neighbourhood:
            return NeighbourhoodParameter(value, step, limits)
        elif self == ParameterType.Range:
            return RangeParameter(value, allowed_range, value_count)
        elif self == ParameterType.Set:
            return SetParameter(value, allowed_values)
        else:
            return None
