import unittest
from typing import TypeVar, List

from src.evolutionary.genotype.parameters.IParameter import IParameter

T = TypeVar("T", int, str, float)


class SetParameter(IParameter):
    def __init__(self, initial_value: T, allowed_values: List[T]):
        self.value: T = initial_value
        self.allowed_values: List[T] = allowed_values
        self.type = "set"

    def get_value(self):
        return self.value

    def set_value(self, value):
        if value not in self.allowed_values:
            raise ValueError("Value not allowed")
        self.value = value

    def get_allowed_values(self) -> list:
        return self.allowed_values

    # Serialize/deserialize
    def to_dict(self) -> dict:
        data = {
            "value": self.value,
            "allowed_values": self.allowed_values,
            "type": self.type,
        }
        return data

    @staticmethod
    def from_dict(data: dict):
        value = data["value"]
        allowed_values = data["allowed_values"]
        return SetParameter(value, allowed_values)


class TestSetParameter(unittest.TestCase):
    def test(self):
        parameter = SetParameter(0, [0, 2, 4, 8])
        self.assertEqual(parameter.get_allowed_values(), [0, 2, 4, 8])
        self.assertEqual(parameter.get_value(), 0)
        parameter.set_value(2)
        self.assertEqual(parameter.get_value(), 2)
