import random

from src.evolutionary.genotype.GenotypeCell import GenotypeModuleBranch
from src.evolutionary.genotype.Genotype import Genotype


def swap_random_layers(genotype1: Genotype, genotype2: Genotype):
    layer_count1 = genotype1.layer_count()
    layer_count2 = genotype2.layer_count()
    if layer_count1 > 0 and layer_count2 > 0:
        layer_indices1 = genotype1.layer_indices()
        layer_indices2 = genotype2.layer_indices()

        random_layer1_index = random.choice(layer_indices1)
        random_layer1 = genotype1.delete_layer(random_layer1_index)

        random_layer2_index = random.choice(layer_indices2)
        random_layer2 = genotype2.delete_layer(random_layer2_index)

        genotype1.add_layer(random_layer2, random_layer1_index)
        genotype2.add_layer(random_layer1, random_layer2_index)


def swap_random_cells(genotype1: Genotype, genotype2: Genotype):
    layer_count1 = genotype1.layer_count()
    layer_count2 = genotype2.layer_count()
    if layer_count1 > 0 and layer_count2 > 0:
        layer_indices1 = genotype1.layer_indices()
        random_layer_index1 = random.choice(layer_indices1)
        layer1 = genotype1.get_layer(random_layer_index1)
        cell_count1 = layer1.cell_count()

        layer_indices2 = genotype2.layer_indices()
        random_layer_index2 = random.choice(layer_indices2)
        layer2 = genotype2.get_layer(random_layer_index2)
        cell_count2 = layer2.cell_count()

        if cell_count1 > 0 and cell_count2 > 0:
            cell_indices1 = layer1.cell_indices()
            random_cell1_index = random.choice(cell_indices1)
            random_cell1 = layer1.get_cell(random_cell1_index)
            random_cell1_hidden_state_indices = layer1.get_cell_hidden_state_indices(
                random_cell1_index
            )

            cell_indices2 = layer2.cell_indices()
            random_cell2_index = random.choice(cell_indices2)
            random_cell2 = layer2.get_cell(random_cell2_index)
            random_cell2_hidden_state_indices = layer2.get_cell_hidden_state_indices(
                random_cell2_index
            )

            layer1.add_cell(
                random_cell1_hidden_state_indices[0],
                random_cell1_hidden_state_indices[1],
                random_cell2,
                random_cell1_index,
            )
            _deleted_cell1 = layer1.delete_cell(
                random_cell1_index + 1
            )  # The old cell will get pushed to i + 1

            layer2.add_cell(
                random_cell2_hidden_state_indices[0],
                random_cell2_hidden_state_indices[1],
                random_cell1,
                random_cell2_index,
            )
            _deleted_cell2 = layer2.delete_cell(
                random_cell2_index + 1
            )  # The old cell will get pushed to i + 1


def swap_random_branches(genotype1: Genotype, genotype2: Genotype):
    layer_count1 = genotype1.layer_count()
    layer_count2 = genotype2.layer_count()
    if layer_count1 > 0 and layer_count2 > 0:
        layer_indices1 = genotype1.layer_indices()
        random_layer_index1 = random.choice(layer_indices1)
        layer1 = genotype1.get_layer(random_layer_index1)
        cell_count1 = layer1.cell_count()

        layer_indices2 = genotype2.layer_indices()
        random_layer_index2 = random.choice(layer_indices2)
        layer2 = genotype2.get_layer(random_layer_index2)
        cell_count2 = layer2.cell_count()

        if cell_count1 > 0 and cell_count2 > 0:
            cell_indices1 = layer1.cell_indices()
            random_cell1_index = random.choice(cell_indices1)
            random_cell1 = layer1.get_cell(random_cell1_index)

            cell_indices2 = layer2.cell_indices()
            random_cell2_index = random.choice(cell_indices2)
            random_cell2 = layer2.get_cell(random_cell2_index)

            branch1_index = random.uniform(0, 1)
            branch1 = (
                GenotypeModuleBranch.LEFT
                if branch1_index < 0.5
                else GenotypeModuleBranch.RIGHT
            )
            modules1 = random_cell1.delete_all_modules(branch1)

            branch2_index = random.uniform(0, 1)
            branch2 = (
                GenotypeModuleBranch.LEFT
                if branch2_index < 0.5
                else GenotypeModuleBranch.RIGHT
            )
            modules2 = random_cell2.delete_all_modules(branch2)

            random_cell1.add_modules(modules2, branch1)
            random_cell2.add_modules(modules1, branch2)
