import math
import random
import unittest

import numpy as np
import torch
import torch.nn as nn


class SimpleEmbedding(nn.Module):
    def __init__(self, embedding_dim, vocabulary_size):
        super(SimpleEmbedding, self).__init__()
        self.embedding = nn.Embedding(vocabulary_size, embedding_dim)
        self.embedding_dim = embedding_dim

    def fix_weights(self, value=0.1):
        nn.init.constant_(self.embedding.weight, value)

    def forward(self, x):
        return self.embedding(x) * math.sqrt(self.embedding_dim)


class TestTimeSeriesEmbedding(unittest.TestCase):
    def test(self):
        random.seed(0)
        torch.manual_seed(0)

        data = np.array([[1], [2]], dtype=np.dtype("i4"))
        x = torch.tensor(data)

        embedding = SimpleEmbedding(4, 10)
        embedding.fix_weights()
        y = embedding.forward(x)

        print(y)
